#ifdef __CPLUSPLUS__
extern "C" {
#endif

#ifndef __GNUC__
#pragma warning(disable: 4275)
#pragma warning(disable: 4101)

#endif
#include "Python.h"
#include "compile.h"
#include "frameobject.h"
#include <complex>
#include <math.h>
#include <string>
#include "scxx/object.h"
#include "scxx/list.h"
#include "scxx/tuple.h"
#include "scxx/dict.h"
#include <iostream>
#include <stdio.h>
#include "numpy/arrayobject.h"




// global None value for use in functions.
namespace py {
object None = object(Py_None);
}

const char* find_type(PyObject* py_obj)
{
    if(py_obj == NULL) return "C NULL value";
    if(PyCallable_Check(py_obj)) return "callable";
    if(PyString_Check(py_obj)) return "string";
    if(PyInt_Check(py_obj)) return "int";
    if(PyFloat_Check(py_obj)) return "float";
    if(PyDict_Check(py_obj)) return "dict";
    if(PyList_Check(py_obj)) return "list";
    if(PyTuple_Check(py_obj)) return "tuple";
    if(PyFile_Check(py_obj)) return "file";
    if(PyModule_Check(py_obj)) return "module";

    //should probably do more intergation (and thinking) on these.
    if(PyCallable_Check(py_obj) && PyInstance_Check(py_obj)) return "callable";
    if(PyInstance_Check(py_obj)) return "instance";
    if(PyCallable_Check(py_obj)) return "callable";
    return "unknown type";
}

void throw_error(PyObject* exc, const char* msg)
{
 //printf("setting python error: %s\n",msg);
  PyErr_SetString(exc, msg);
  //printf("throwing error\n");
  throw 1;
}

void handle_bad_type(PyObject* py_obj, const char* good_type, const char* var_name)
{
    char msg[500];
    sprintf(msg,"received '%s' type instead of '%s' for variable '%s'",
            find_type(py_obj),good_type,var_name);
    throw_error(PyExc_TypeError,msg);
}

void handle_conversion_error(PyObject* py_obj, const char* good_type, const char* var_name)
{
    char msg[500];
    sprintf(msg,"Conversion Error:, received '%s' type instead of '%s' for variable '%s'",
            find_type(py_obj),good_type,var_name);
    throw_error(PyExc_TypeError,msg);
}


class int_handler
{
public:
    int convert_to_int(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        
        if (!py_obj || !PyInt_Check(py_obj))
            handle_conversion_error(py_obj,"int", name);
        return (int) PyInt_AsLong(py_obj);
    }

    int py_to_int(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyInt_Check(py_obj))
            handle_bad_type(py_obj,"int", name);
        
        return (int) PyInt_AsLong(py_obj);
    }
};

int_handler x__int_handler = int_handler();
#define convert_to_int(py_obj,name) \
        x__int_handler.convert_to_int(py_obj,name)
#define py_to_int(py_obj,name) \
        x__int_handler.py_to_int(py_obj,name)


PyObject* int_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}


class float_handler
{
public:
    double convert_to_float(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        
        if (!py_obj || !PyFloat_Check(py_obj))
            handle_conversion_error(py_obj,"float", name);
        return PyFloat_AsDouble(py_obj);
    }

    double py_to_float(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyFloat_Check(py_obj))
            handle_bad_type(py_obj,"float", name);
        
        return PyFloat_AsDouble(py_obj);
    }
};

float_handler x__float_handler = float_handler();
#define convert_to_float(py_obj,name) \
        x__float_handler.convert_to_float(py_obj,name)
#define py_to_float(py_obj,name) \
        x__float_handler.py_to_float(py_obj,name)


PyObject* float_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}


class complex_handler
{
public:
    std::complex<double> convert_to_complex(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        
        if (!py_obj || !PyComplex_Check(py_obj))
            handle_conversion_error(py_obj,"complex", name);
        return std::complex<double>(PyComplex_RealAsDouble(py_obj),PyComplex_ImagAsDouble(py_obj));
    }

    std::complex<double> py_to_complex(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyComplex_Check(py_obj))
            handle_bad_type(py_obj,"complex", name);
        
        return std::complex<double>(PyComplex_RealAsDouble(py_obj),PyComplex_ImagAsDouble(py_obj));
    }
};

complex_handler x__complex_handler = complex_handler();
#define convert_to_complex(py_obj,name) \
        x__complex_handler.convert_to_complex(py_obj,name)
#define py_to_complex(py_obj,name) \
        x__complex_handler.py_to_complex(py_obj,name)


PyObject* complex_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}


class unicode_handler
{
public:
    Py_UNICODE* convert_to_unicode(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        Py_XINCREF(py_obj);
        if (!py_obj || !PyUnicode_Check(py_obj))
            handle_conversion_error(py_obj,"unicode", name);
        return PyUnicode_AS_UNICODE(py_obj);
    }

    Py_UNICODE* py_to_unicode(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyUnicode_Check(py_obj))
            handle_bad_type(py_obj,"unicode", name);
        Py_XINCREF(py_obj);
        return PyUnicode_AS_UNICODE(py_obj);
    }
};

unicode_handler x__unicode_handler = unicode_handler();
#define convert_to_unicode(py_obj,name) \
        x__unicode_handler.convert_to_unicode(py_obj,name)
#define py_to_unicode(py_obj,name) \
        x__unicode_handler.py_to_unicode(py_obj,name)


PyObject* unicode_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}


class string_handler
{
public:
    std::string convert_to_string(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        Py_XINCREF(py_obj);
        if (!py_obj || !PyString_Check(py_obj))
            handle_conversion_error(py_obj,"string", name);
        return std::string(PyString_AsString(py_obj));
    }

    std::string py_to_string(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyString_Check(py_obj))
            handle_bad_type(py_obj,"string", name);
        Py_XINCREF(py_obj);
        return std::string(PyString_AsString(py_obj));
    }
};

string_handler x__string_handler = string_handler();
#define convert_to_string(py_obj,name) \
        x__string_handler.convert_to_string(py_obj,name)
#define py_to_string(py_obj,name) \
        x__string_handler.py_to_string(py_obj,name)


               PyObject* string_to_py(std::string s)
               {
                   return PyString_FromString(s.c_str());
               }
               
class list_handler
{
public:
    py::list convert_to_list(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        
        if (!py_obj || !PyList_Check(py_obj))
            handle_conversion_error(py_obj,"list", name);
        return py::list(py_obj);
    }

    py::list py_to_list(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyList_Check(py_obj))
            handle_bad_type(py_obj,"list", name);
        
        return py::list(py_obj);
    }
};

list_handler x__list_handler = list_handler();
#define convert_to_list(py_obj,name) \
        x__list_handler.convert_to_list(py_obj,name)
#define py_to_list(py_obj,name) \
        x__list_handler.py_to_list(py_obj,name)


PyObject* list_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}


class dict_handler
{
public:
    py::dict convert_to_dict(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        
        if (!py_obj || !PyDict_Check(py_obj))
            handle_conversion_error(py_obj,"dict", name);
        return py::dict(py_obj);
    }

    py::dict py_to_dict(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyDict_Check(py_obj))
            handle_bad_type(py_obj,"dict", name);
        
        return py::dict(py_obj);
    }
};

dict_handler x__dict_handler = dict_handler();
#define convert_to_dict(py_obj,name) \
        x__dict_handler.convert_to_dict(py_obj,name)
#define py_to_dict(py_obj,name) \
        x__dict_handler.py_to_dict(py_obj,name)


PyObject* dict_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}


class tuple_handler
{
public:
    py::tuple convert_to_tuple(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        
        if (!py_obj || !PyTuple_Check(py_obj))
            handle_conversion_error(py_obj,"tuple", name);
        return py::tuple(py_obj);
    }

    py::tuple py_to_tuple(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyTuple_Check(py_obj))
            handle_bad_type(py_obj,"tuple", name);
        
        return py::tuple(py_obj);
    }
};

tuple_handler x__tuple_handler = tuple_handler();
#define convert_to_tuple(py_obj,name) \
        x__tuple_handler.convert_to_tuple(py_obj,name)
#define py_to_tuple(py_obj,name) \
        x__tuple_handler.py_to_tuple(py_obj,name)


PyObject* tuple_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}


class file_handler
{
public:
    FILE* convert_to_file(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        Py_XINCREF(py_obj);
        if (!py_obj || !PyFile_Check(py_obj))
            handle_conversion_error(py_obj,"file", name);
        return PyFile_AsFile(py_obj);
    }

    FILE* py_to_file(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyFile_Check(py_obj))
            handle_bad_type(py_obj,"file", name);
        Py_XINCREF(py_obj);
        return PyFile_AsFile(py_obj);
    }
};

file_handler x__file_handler = file_handler();
#define convert_to_file(py_obj,name) \
        x__file_handler.convert_to_file(py_obj,name)
#define py_to_file(py_obj,name) \
        x__file_handler.py_to_file(py_obj,name)


               PyObject* file_to_py(FILE* file, const char* name,
                                    const char* mode)
               {
                   return (PyObject*) PyFile_FromFile(file,
                     const_cast<char*>(name),
                     const_cast<char*>(mode), fclose);
               }
               
class instance_handler
{
public:
    py::object convert_to_instance(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        
        if (!py_obj || !PyInstance_Check(py_obj))
            handle_conversion_error(py_obj,"instance", name);
        return py::object(py_obj);
    }

    py::object py_to_instance(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyInstance_Check(py_obj))
            handle_bad_type(py_obj,"instance", name);
        
        return py::object(py_obj);
    }
};

instance_handler x__instance_handler = instance_handler();
#define convert_to_instance(py_obj,name) \
        x__instance_handler.convert_to_instance(py_obj,name)
#define py_to_instance(py_obj,name) \
        x__instance_handler.py_to_instance(py_obj,name)


PyObject* instance_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}


class numpy_size_handler
{
public:
    void conversion_numpy_check_size(PyArrayObject* arr_obj, int Ndims,
                                     const char* name)
    {
        if (arr_obj->nd != Ndims)
        {
            char msg[500];
            sprintf(msg,"Conversion Error: received '%d' dimensional array instead of '%d' dimensional array for variable '%s'",
                    arr_obj->nd,Ndims,name);
            throw_error(PyExc_TypeError,msg);
        }
    }

    void numpy_check_size(PyArrayObject* arr_obj, int Ndims, const char* name)
    {
        if (arr_obj->nd != Ndims)
        {
            char msg[500];
            sprintf(msg,"received '%d' dimensional array instead of '%d' dimensional array for variable '%s'",
                    arr_obj->nd,Ndims,name);
            throw_error(PyExc_TypeError,msg);
        }
    }
};

numpy_size_handler x__numpy_size_handler = numpy_size_handler();
#define conversion_numpy_check_size x__numpy_size_handler.conversion_numpy_check_size
#define numpy_check_size x__numpy_size_handler.numpy_check_size


class numpy_type_handler
{
public:
    void conversion_numpy_check_type(PyArrayObject* arr_obj, int numeric_type,
                                     const char* name)
    {
        // Make sure input has correct numeric type.
        int arr_type = arr_obj->descr->type_num;
        if (PyTypeNum_ISEXTENDED(numeric_type))
        {
        char msg[80];
        sprintf(msg, "Conversion Error: extended types not supported for variable '%s'",
                name);
        throw_error(PyExc_TypeError, msg);
        }
        if (!PyArray_EquivTypenums(arr_type, numeric_type))
        {

        const char* type_names[23] = {"bool", "byte", "ubyte","short", "ushort",
                                "int", "uint", "long", "ulong", "longlong", "ulonglong",
                                "float", "double", "longdouble", "cfloat", "cdouble",
                                "clongdouble", "object", "string", "unicode", "void", "ntype",
                                "unknown"};
        char msg[500];
        sprintf(msg,"Conversion Error: received '%s' typed array instead of '%s' typed array for variable '%s'",
                type_names[arr_type],type_names[numeric_type],name);
        throw_error(PyExc_TypeError,msg);
        }
    }

    void numpy_check_type(PyArrayObject* arr_obj, int numeric_type, const char* name)
    {
        // Make sure input has correct numeric type.
        int arr_type = arr_obj->descr->type_num;
        if (PyTypeNum_ISEXTENDED(numeric_type))
        {
        char msg[80];
        sprintf(msg, "Conversion Error: extended types not supported for variable '%s'",
                name);
        throw_error(PyExc_TypeError, msg);
        }
        if (!PyArray_EquivTypenums(arr_type, numeric_type))
        {
            const char* type_names[23] = {"bool", "byte", "ubyte","short", "ushort",
                                    "int", "uint", "long", "ulong", "longlong", "ulonglong",
                                    "float", "double", "longdouble", "cfloat", "cdouble",
                                    "clongdouble", "object", "string", "unicode", "void", "ntype",
                                    "unknown"};
            char msg[500];
            sprintf(msg,"received '%s' typed array instead of '%s' typed array for variable '%s'",
                    type_names[arr_type],type_names[numeric_type],name);
            throw_error(PyExc_TypeError,msg);
        }
    }
};

numpy_type_handler x__numpy_type_handler = numpy_type_handler();
#define conversion_numpy_check_type x__numpy_type_handler.conversion_numpy_check_type
#define numpy_check_type x__numpy_type_handler.numpy_check_type


class numpy_handler
{
public:
    PyArrayObject* convert_to_numpy(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        Py_XINCREF(py_obj);
        if (!py_obj || !PyArray_Check(py_obj))
            handle_conversion_error(py_obj,"numpy", name);
        return (PyArrayObject*) py_obj;
    }

    PyArrayObject* py_to_numpy(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !PyArray_Check(py_obj))
            handle_bad_type(py_obj,"numpy", name);
        Py_XINCREF(py_obj);
        return (PyArrayObject*) py_obj;
    }
};

numpy_handler x__numpy_handler = numpy_handler();
#define convert_to_numpy(py_obj,name) \
        x__numpy_handler.convert_to_numpy(py_obj,name)
#define py_to_numpy(py_obj,name) \
        x__numpy_handler.py_to_numpy(py_obj,name)


PyObject* numpy_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}


class catchall_handler
{
public:
    py::object convert_to_catchall(PyObject* py_obj, const char* name)
    {
        // Incref occurs even if conversion fails so that
        // the decref in cleanup_code has a matching incref.
        
        if (!py_obj || !(py_obj))
            handle_conversion_error(py_obj,"catchall", name);
        return py::object(py_obj);
    }

    py::object py_to_catchall(PyObject* py_obj, const char* name)
    {
        // !! Pretty sure INCREF should only be called on success since
        // !! py_to_xxx is used by the user -- not the code generator.
        if (!py_obj || !(py_obj))
            handle_bad_type(py_obj,"catchall", name);
        
        return py::object(py_obj);
    }
};

catchall_handler x__catchall_handler = catchall_handler();
#define convert_to_catchall(py_obj,name) \
        x__catchall_handler.convert_to_catchall(py_obj,name)
#define py_to_catchall(py_obj,name) \
        x__catchall_handler.py_to_catchall(py_obj,name)


PyObject* catchall_to_py(PyObject* obj)
{
    return (PyObject*) obj;
}



static PyObject* write_array_i(PyObject*self, PyObject* args, PyObject* kywds)
{
    py::object return_val;
    int exception_occurred = 0;
    PyObject *py_local_dict = NULL;
    static const char *kwlist[] = {"file","ary","format","element_separator","line_separator","local_dict", NULL};
    PyObject *py_file, *py_ary, *py_format, *py_element_separator, *py_line_separator;
    int file_used, ary_used, format_used, element_separator_used, line_separator_used;
    py_file = py_ary = py_format = py_element_separator = py_line_separator = NULL;
    file_used= ary_used= format_used= element_separator_used= line_separator_used = 0;
    
    if(!PyArg_ParseTupleAndKeywords(args,kywds,"OOOOO|O:write_array_i",const_cast<char**>(kwlist),&py_file, &py_ary, &py_format, &py_element_separator, &py_line_separator, &py_local_dict))
       return NULL;
    try                              
    {                                
        py_file = py_file;
        FILE* file = convert_to_file(py_file,"file");
        file_used = 1;
        py_ary = py_ary;
        PyArrayObject* ary_array = convert_to_numpy(py_ary,"ary");
        conversion_numpy_check_type(ary_array,PyArray_LONG,"ary");
        #define ARY1(i) (*((long*)(ary_array->data + (i)*Sary[0])))
        #define ARY2(i,j) (*((long*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1])))
        #define ARY3(i,j,k) (*((long*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2])))
        #define ARY4(i,j,k,l) (*((long*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2] + (l)*Sary[3])))
        npy_intp* Nary = ary_array->dimensions;
        npy_intp* Sary = ary_array->strides;
        int Dary = ary_array->nd;
        long* ary = (long*) ary_array->data;
        ary_used = 1;
        py_format = py_format;
        std::string format = convert_to_string(py_format,"format");
        format_used = 1;
        py_element_separator = py_element_separator;
        std::string element_separator = convert_to_string(py_element_separator,"element_separator");
        element_separator_used = 1;
        py_line_separator = py_line_separator;
        std::string line_separator = convert_to_string(py_line_separator,"line_separator");
        line_separator_used = 1;
        /*<function call here>*/     
        int element_size = sizeof(*ary);                     
        int row_offset = Sary[0]/element_size;               
        int col_offset = Sary[1]/element_size;               
                                                             
        int* row = (int*) ary;                     
        for (int i=0;i<Nary[0];i++)                          
        {                                                    
            int* element = row;                         
            for (int j=0;j<Nary[1];j++)                      
            {                                                
                fprintf(file, format.c_str(), *element);  
                fprintf(file, element_separator.c_str());    
                element += col_offset;                       
            }                                                
            fprintf(file, line_separator.c_str());           
            row += row_offset;                               
        }                                                    
        if(py_local_dict)                                  
        {                                                  
            py::dict local_dict = py::dict(py_local_dict); 
        }                                                  
    
    }                                
    catch(...)                       
    {                                
        return_val =  py::object();      
        exception_occurred = 1;       
    }                                
    /*cleanup code*/                     
    if(file_used)
    {
        Py_XDECREF(py_file);
    }
    if(ary_used)
    {
        Py_XDECREF(py_ary);
        #undef ARY1
        #undef ARY2
        #undef ARY3
        #undef ARY4
    }
    if(format_used)
    {
        Py_XDECREF(py_format);
    }
    if(element_separator_used)
    {
        Py_XDECREF(py_element_separator);
    }
    if(line_separator_used)
    {
        Py_XDECREF(py_line_separator);
    }
    if(!(PyObject*)return_val && !exception_occurred)
    {
                                  
        return_val = Py_None;            
    }
                                  
    return return_val.disown();           
}                                
static PyObject* write_array_ui(PyObject*self, PyObject* args, PyObject* kywds)
{
    py::object return_val;
    int exception_occurred = 0;
    PyObject *py_local_dict = NULL;
    static const char *kwlist[] = {"file","ary","format","element_separator","line_separator","local_dict", NULL};
    PyObject *py_file, *py_ary, *py_format, *py_element_separator, *py_line_separator;
    int file_used, ary_used, format_used, element_separator_used, line_separator_used;
    py_file = py_ary = py_format = py_element_separator = py_line_separator = NULL;
    file_used= ary_used= format_used= element_separator_used= line_separator_used = 0;
    
    if(!PyArg_ParseTupleAndKeywords(args,kywds,"OOOOO|O:write_array_ui",const_cast<char**>(kwlist),&py_file, &py_ary, &py_format, &py_element_separator, &py_line_separator, &py_local_dict))
       return NULL;
    try                              
    {                                
        py_file = py_file;
        FILE* file = convert_to_file(py_file,"file");
        file_used = 1;
        py_ary = py_ary;
        PyArrayObject* ary_array = convert_to_numpy(py_ary,"ary");
        conversion_numpy_check_type(ary_array,PyArray_ULONG,"ary");
        #define ARY1(i) (*((npy_ulong*)(ary_array->data + (i)*Sary[0])))
        #define ARY2(i,j) (*((npy_ulong*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1])))
        #define ARY3(i,j,k) (*((npy_ulong*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2])))
        #define ARY4(i,j,k,l) (*((npy_ulong*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2] + (l)*Sary[3])))
        npy_intp* Nary = ary_array->dimensions;
        npy_intp* Sary = ary_array->strides;
        int Dary = ary_array->nd;
        npy_ulong* ary = (npy_ulong*) ary_array->data;
        ary_used = 1;
        py_format = py_format;
        std::string format = convert_to_string(py_format,"format");
        format_used = 1;
        py_element_separator = py_element_separator;
        std::string element_separator = convert_to_string(py_element_separator,"element_separator");
        element_separator_used = 1;
        py_line_separator = py_line_separator;
        std::string line_separator = convert_to_string(py_line_separator,"line_separator");
        line_separator_used = 1;
        /*<function call here>*/     
        int element_size = sizeof(*ary);                     
        int row_offset = Sary[0]/element_size;               
        int col_offset = Sary[1]/element_size;               
                                                             
        unsigned int* row = (unsigned int*) ary;                     
        for (int i=0;i<Nary[0];i++)                          
        {                                                    
            unsigned int* element = row;                         
            for (int j=0;j<Nary[1];j++)                      
            {                                                
                fprintf(file, format.c_str(), *element);  
                fprintf(file, element_separator.c_str());    
                element += col_offset;                       
            }                                                
            fprintf(file, line_separator.c_str());           
            row += row_offset;                               
        }                                                    
        if(py_local_dict)                                  
        {                                                  
            py::dict local_dict = py::dict(py_local_dict); 
        }                                                  
    
    }                                
    catch(...)                       
    {                                
        return_val =  py::object();      
        exception_occurred = 1;       
    }                                
    /*cleanup code*/                     
    if(file_used)
    {
        Py_XDECREF(py_file);
    }
    if(ary_used)
    {
        Py_XDECREF(py_ary);
        #undef ARY1
        #undef ARY2
        #undef ARY3
        #undef ARY4
    }
    if(format_used)
    {
        Py_XDECREF(py_format);
    }
    if(element_separator_used)
    {
        Py_XDECREF(py_element_separator);
    }
    if(line_separator_used)
    {
        Py_XDECREF(py_line_separator);
    }
    if(!(PyObject*)return_val && !exception_occurred)
    {
                                  
        return_val = Py_None;            
    }
                                  
    return return_val.disown();           
}                                
static PyObject* write_array_f(PyObject*self, PyObject* args, PyObject* kywds)
{
    py::object return_val;
    int exception_occurred = 0;
    PyObject *py_local_dict = NULL;
    static const char *kwlist[] = {"file","ary","format","element_separator","line_separator","local_dict", NULL};
    PyObject *py_file, *py_ary, *py_format, *py_element_separator, *py_line_separator;
    int file_used, ary_used, format_used, element_separator_used, line_separator_used;
    py_file = py_ary = py_format = py_element_separator = py_line_separator = NULL;
    file_used= ary_used= format_used= element_separator_used= line_separator_used = 0;
    
    if(!PyArg_ParseTupleAndKeywords(args,kywds,"OOOOO|O:write_array_f",const_cast<char**>(kwlist),&py_file, &py_ary, &py_format, &py_element_separator, &py_line_separator, &py_local_dict))
       return NULL;
    try                              
    {                                
        py_file = py_file;
        FILE* file = convert_to_file(py_file,"file");
        file_used = 1;
        py_ary = py_ary;
        PyArrayObject* ary_array = convert_to_numpy(py_ary,"ary");
        conversion_numpy_check_type(ary_array,PyArray_FLOAT,"ary");
        #define ARY1(i) (*((float*)(ary_array->data + (i)*Sary[0])))
        #define ARY2(i,j) (*((float*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1])))
        #define ARY3(i,j,k) (*((float*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2])))
        #define ARY4(i,j,k,l) (*((float*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2] + (l)*Sary[3])))
        npy_intp* Nary = ary_array->dimensions;
        npy_intp* Sary = ary_array->strides;
        int Dary = ary_array->nd;
        float* ary = (float*) ary_array->data;
        ary_used = 1;
        py_format = py_format;
        std::string format = convert_to_string(py_format,"format");
        format_used = 1;
        py_element_separator = py_element_separator;
        std::string element_separator = convert_to_string(py_element_separator,"element_separator");
        element_separator_used = 1;
        py_line_separator = py_line_separator;
        std::string line_separator = convert_to_string(py_line_separator,"line_separator");
        line_separator_used = 1;
        /*<function call here>*/     
        int element_size = sizeof(*ary);                     
        int row_offset = Sary[0]/element_size;               
        int col_offset = Sary[1]/element_size;               
                                                             
        float* row = (float*) ary;                     
        for (int i=0;i<Nary[0];i++)                          
        {                                                    
            float* element = row;                         
            for (int j=0;j<Nary[1];j++)                      
            {                                                
                fprintf(file, format.c_str(), *element);  
                fprintf(file, element_separator.c_str());    
                element += col_offset;                       
            }                                                
            fprintf(file, line_separator.c_str());           
            row += row_offset;                               
        }                                                    
        if(py_local_dict)                                  
        {                                                  
            py::dict local_dict = py::dict(py_local_dict); 
        }                                                  
    
    }                                
    catch(...)                       
    {                                
        return_val =  py::object();      
        exception_occurred = 1;       
    }                                
    /*cleanup code*/                     
    if(file_used)
    {
        Py_XDECREF(py_file);
    }
    if(ary_used)
    {
        Py_XDECREF(py_ary);
        #undef ARY1
        #undef ARY2
        #undef ARY3
        #undef ARY4
    }
    if(format_used)
    {
        Py_XDECREF(py_format);
    }
    if(element_separator_used)
    {
        Py_XDECREF(py_element_separator);
    }
    if(line_separator_used)
    {
        Py_XDECREF(py_line_separator);
    }
    if(!(PyObject*)return_val && !exception_occurred)
    {
                                  
        return_val = Py_None;            
    }
                                  
    return return_val.disown();           
}                                
static PyObject* write_array_d(PyObject*self, PyObject* args, PyObject* kywds)
{
    py::object return_val;
    int exception_occurred = 0;
    PyObject *py_local_dict = NULL;
    static const char *kwlist[] = {"file","ary","format","element_separator","line_separator","local_dict", NULL};
    PyObject *py_file, *py_ary, *py_format, *py_element_separator, *py_line_separator;
    int file_used, ary_used, format_used, element_separator_used, line_separator_used;
    py_file = py_ary = py_format = py_element_separator = py_line_separator = NULL;
    file_used= ary_used= format_used= element_separator_used= line_separator_used = 0;
    
    if(!PyArg_ParseTupleAndKeywords(args,kywds,"OOOOO|O:write_array_d",const_cast<char**>(kwlist),&py_file, &py_ary, &py_format, &py_element_separator, &py_line_separator, &py_local_dict))
       return NULL;
    try                              
    {                                
        py_file = py_file;
        FILE* file = convert_to_file(py_file,"file");
        file_used = 1;
        py_ary = py_ary;
        PyArrayObject* ary_array = convert_to_numpy(py_ary,"ary");
        conversion_numpy_check_type(ary_array,PyArray_DOUBLE,"ary");
        #define ARY1(i) (*((double*)(ary_array->data + (i)*Sary[0])))
        #define ARY2(i,j) (*((double*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1])))
        #define ARY3(i,j,k) (*((double*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2])))
        #define ARY4(i,j,k,l) (*((double*)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2] + (l)*Sary[3])))
        npy_intp* Nary = ary_array->dimensions;
        npy_intp* Sary = ary_array->strides;
        int Dary = ary_array->nd;
        double* ary = (double*) ary_array->data;
        ary_used = 1;
        py_format = py_format;
        std::string format = convert_to_string(py_format,"format");
        format_used = 1;
        py_element_separator = py_element_separator;
        std::string element_separator = convert_to_string(py_element_separator,"element_separator");
        element_separator_used = 1;
        py_line_separator = py_line_separator;
        std::string line_separator = convert_to_string(py_line_separator,"line_separator");
        line_separator_used = 1;
        /*<function call here>*/     
        int element_size = sizeof(*ary);                     
        int row_offset = Sary[0]/element_size;               
        int col_offset = Sary[1]/element_size;               
                                                             
        double* row = (double*) ary;                     
        for (int i=0;i<Nary[0];i++)                          
        {                                                    
            double* element = row;                         
            for (int j=0;j<Nary[1];j++)                      
            {                                                
                fprintf(file, format.c_str(), *element);  
                fprintf(file, element_separator.c_str());    
                element += col_offset;                       
            }                                                
            fprintf(file, line_separator.c_str());           
            row += row_offset;                               
        }                                                    
        if(py_local_dict)                                  
        {                                                  
            py::dict local_dict = py::dict(py_local_dict); 
        }                                                  
    
    }                                
    catch(...)                       
    {                                
        return_val =  py::object();      
        exception_occurred = 1;       
    }                                
    /*cleanup code*/                     
    if(file_used)
    {
        Py_XDECREF(py_file);
    }
    if(ary_used)
    {
        Py_XDECREF(py_ary);
        #undef ARY1
        #undef ARY2
        #undef ARY3
        #undef ARY4
    }
    if(format_used)
    {
        Py_XDECREF(py_format);
    }
    if(element_separator_used)
    {
        Py_XDECREF(py_element_separator);
    }
    if(line_separator_used)
    {
        Py_XDECREF(py_line_separator);
    }
    if(!(PyObject*)return_val && !exception_occurred)
    {
                                  
        return_val = Py_None;            
    }
                                  
    return return_val.disown();           
}                                
static PyObject* write_array_F(PyObject*self, PyObject* args, PyObject* kywds)
{
    py::object return_val;
    int exception_occurred = 0;
    PyObject *py_local_dict = NULL;
    static const char *kwlist[] = {"file","ary","format","element_separator","line_separator","local_dict", NULL};
    PyObject *py_file, *py_ary, *py_format, *py_element_separator, *py_line_separator;
    int file_used, ary_used, format_used, element_separator_used, line_separator_used;
    py_file = py_ary = py_format = py_element_separator = py_line_separator = NULL;
    file_used= ary_used= format_used= element_separator_used= line_separator_used = 0;
    
    if(!PyArg_ParseTupleAndKeywords(args,kywds,"OOOOO|O:write_array_F",const_cast<char**>(kwlist),&py_file, &py_ary, &py_format, &py_element_separator, &py_line_separator, &py_local_dict))
       return NULL;
    try                              
    {                                
        py_file = py_file;
        FILE* file = convert_to_file(py_file,"file");
        file_used = 1;
        py_ary = py_ary;
        PyArrayObject* ary_array = convert_to_numpy(py_ary,"ary");
        conversion_numpy_check_type(ary_array,PyArray_CFLOAT,"ary");
        #define ARY1(i) (*((std::complex<float> *)(ary_array->data + (i)*Sary[0])))
        #define ARY2(i,j) (*((std::complex<float> *)(ary_array->data + (i)*Sary[0] + (j)*Sary[1])))
        #define ARY3(i,j,k) (*((std::complex<float> *)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2])))
        #define ARY4(i,j,k,l) (*((std::complex<float> *)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2] + (l)*Sary[3])))
        npy_intp* Nary = ary_array->dimensions;
        npy_intp* Sary = ary_array->strides;
        int Dary = ary_array->nd;
        std::complex<float> * ary = (std::complex<float> *) ary_array->data;
        ary_used = 1;
        py_format = py_format;
        std::string format = convert_to_string(py_format,"format");
        format_used = 1;
        py_element_separator = py_element_separator;
        std::string element_separator = convert_to_string(py_element_separator,"element_separator");
        element_separator_used = 1;
        py_line_separator = py_line_separator;
        std::string line_separator = convert_to_string(py_line_separator,"line_separator");
        line_separator_used = 1;
        /*<function call here>*/     
        int element_size = sizeof(*ary);                     
        int row_offset = Sary[0]/element_size;               
        int col_offset = Sary[1]/element_size;               
                                                             
        std::complex<float>* row = (std::complex<float>*) ary;                     
        for (int i=0;i<Nary[0];i++)                          
        {                                                    
            std::complex<float>* element = row;                         
            for (int j=0;j<Nary[1];j++)                      
            {                                                
                fprintf(file, format.c_str(), element->real(),element->imag());  
                fprintf(file, element_separator.c_str());    
                element += col_offset;                       
            }                                                
            fprintf(file, line_separator.c_str());           
            row += row_offset;                               
        }                                                    
        if(py_local_dict)                                  
        {                                                  
            py::dict local_dict = py::dict(py_local_dict); 
        }                                                  
    
    }                                
    catch(...)                       
    {                                
        return_val =  py::object();      
        exception_occurred = 1;       
    }                                
    /*cleanup code*/                     
    if(file_used)
    {
        Py_XDECREF(py_file);
    }
    if(ary_used)
    {
        Py_XDECREF(py_ary);
        #undef ARY1
        #undef ARY2
        #undef ARY3
        #undef ARY4
    }
    if(format_used)
    {
        Py_XDECREF(py_format);
    }
    if(element_separator_used)
    {
        Py_XDECREF(py_element_separator);
    }
    if(line_separator_used)
    {
        Py_XDECREF(py_line_separator);
    }
    if(!(PyObject*)return_val && !exception_occurred)
    {
                                  
        return_val = Py_None;            
    }
                                  
    return return_val.disown();           
}                                
static PyObject* write_array_D(PyObject*self, PyObject* args, PyObject* kywds)
{
    py::object return_val;
    int exception_occurred = 0;
    PyObject *py_local_dict = NULL;
    static const char *kwlist[] = {"file","ary","format","element_separator","line_separator","local_dict", NULL};
    PyObject *py_file, *py_ary, *py_format, *py_element_separator, *py_line_separator;
    int file_used, ary_used, format_used, element_separator_used, line_separator_used;
    py_file = py_ary = py_format = py_element_separator = py_line_separator = NULL;
    file_used= ary_used= format_used= element_separator_used= line_separator_used = 0;
    
    if(!PyArg_ParseTupleAndKeywords(args,kywds,"OOOOO|O:write_array_D",const_cast<char**>(kwlist),&py_file, &py_ary, &py_format, &py_element_separator, &py_line_separator, &py_local_dict))
       return NULL;
    try                              
    {                                
        py_file = py_file;
        FILE* file = convert_to_file(py_file,"file");
        file_used = 1;
        py_ary = py_ary;
        PyArrayObject* ary_array = convert_to_numpy(py_ary,"ary");
        conversion_numpy_check_type(ary_array,PyArray_CDOUBLE,"ary");
        #define ARY1(i) (*((std::complex<double> *)(ary_array->data + (i)*Sary[0])))
        #define ARY2(i,j) (*((std::complex<double> *)(ary_array->data + (i)*Sary[0] + (j)*Sary[1])))
        #define ARY3(i,j,k) (*((std::complex<double> *)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2])))
        #define ARY4(i,j,k,l) (*((std::complex<double> *)(ary_array->data + (i)*Sary[0] + (j)*Sary[1] + (k)*Sary[2] + (l)*Sary[3])))
        npy_intp* Nary = ary_array->dimensions;
        npy_intp* Sary = ary_array->strides;
        int Dary = ary_array->nd;
        std::complex<double> * ary = (std::complex<double> *) ary_array->data;
        ary_used = 1;
        py_format = py_format;
        std::string format = convert_to_string(py_format,"format");
        format_used = 1;
        py_element_separator = py_element_separator;
        std::string element_separator = convert_to_string(py_element_separator,"element_separator");
        element_separator_used = 1;
        py_line_separator = py_line_separator;
        std::string line_separator = convert_to_string(py_line_separator,"line_separator");
        line_separator_used = 1;
        /*<function call here>*/     
        int element_size = sizeof(*ary);                     
        int row_offset = Sary[0]/element_size;               
        int col_offset = Sary[1]/element_size;               
                                                             
        std::complex<double>* row = (std::complex<double>*) ary;                     
        for (int i=0;i<Nary[0];i++)                          
        {                                                    
            std::complex<double>* element = row;                         
            for (int j=0;j<Nary[1];j++)                      
            {                                                
                fprintf(file, format.c_str(), element->real(),element->imag());  
                fprintf(file, element_separator.c_str());    
                element += col_offset;                       
            }                                                
            fprintf(file, line_separator.c_str());           
            row += row_offset;                               
        }                                                    
        if(py_local_dict)                                  
        {                                                  
            py::dict local_dict = py::dict(py_local_dict); 
        }                                                  
    
    }                                
    catch(...)                       
    {                                
        return_val =  py::object();      
        exception_occurred = 1;       
    }                                
    /*cleanup code*/                     
    if(file_used)
    {
        Py_XDECREF(py_file);
    }
    if(ary_used)
    {
        Py_XDECREF(py_ary);
        #undef ARY1
        #undef ARY2
        #undef ARY3
        #undef ARY4
    }
    if(format_used)
    {
        Py_XDECREF(py_format);
    }
    if(element_separator_used)
    {
        Py_XDECREF(py_element_separator);
    }
    if(line_separator_used)
    {
        Py_XDECREF(py_line_separator);
    }
    if(!(PyObject*)return_val && !exception_occurred)
    {
                                  
        return_val = Py_None;            
    }
                                  
    return return_val.disown();           
}                                


static PyMethodDef compiled_methods[] = 
{
    {"write_array_i",(PyCFunction)write_array_i , METH_VARARGS|METH_KEYWORDS},
    {"write_array_ui",(PyCFunction)write_array_ui , METH_VARARGS|METH_KEYWORDS},
    {"write_array_f",(PyCFunction)write_array_f , METH_VARARGS|METH_KEYWORDS},
    {"write_array_d",(PyCFunction)write_array_d , METH_VARARGS|METH_KEYWORDS},
    {"write_array_F",(PyCFunction)write_array_F , METH_VARARGS|METH_KEYWORDS},
    {"write_array_D",(PyCFunction)write_array_D , METH_VARARGS|METH_KEYWORDS},
    {NULL,      NULL}        /* Sentinel */
};

PyMODINIT_FUNC init_write_array(void)
{
    
    Py_Initialize();
    import_array();
    PyImport_ImportModule("numpy");
    (void) Py_InitModule("_write_array", compiled_methods);
}

#ifdef __CPLUSCPLUS__
}
#endif
